@@ -19,6 +19,8 @@
 
 
 #include "mainwindow.h"
+#include <QDir>
+#include <QFileInfo>
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QImage>
@@ -28,13 +30,14 @@
 #include <QPainter>
 #include <QPrintDialog>
 #include <QPrinter>
-#include <QDebug>
 #include <QWheelEvent>
 #include <QMouseEvent>
 #include <QTimer>
 #include <QShortcut>
 #include <QDockWidget>
 #include <QScrollBar>
+#include <QDesktopWidget>
+#include <QGraphicsSvgItem>
 #include "application.h"
 #include <libfm-qt/path.h>
 #include <libfm-qt/folderview.h>
@@ -45,21 +48,21 @@ using namespace LxImage;
 
 MainWindow::MainWindow():
   QMainWindow(),
-  currentFile_(NULL),
-  slideShowTimer_(NULL),
-  // currentFileInfo_(NULL),
-  loadJob_(NULL),
-  saveJob_(NULL),
-  folder_(NULL),
-  folderPath_(NULL),
+  contextMenu_(new QMenu(this)),
+  slideShowTimer_(nullptr),
+  image_(),
+  currentFile_(nullptr),
+  // currentFileInfo_(nullptr),
+  imageModified_(false),
+  folder_(nullptr),
+  folderPath_(nullptr),
   folderModel_(new Fm::FolderModel()),
   proxyModel_(new Fm::ProxyFolderModel()),
   modelFilter_(new ModelFilter()),
-  imageModified_(false),
-  contextMenu_(new QMenu(this)),
-  thumbnailsDock_(NULL),
-  thumbnailsView_(NULL),
-  image_() {
+  thumbnailsDock_(nullptr),
+  thumbnailsView_(nullptr),
+  loadJob_(nullptr),
+  saveJob_(nullptr) {
 
   setAttribute(Qt::WA_DeleteOnClose); // FIXME: check if current image is saved before close
 
@@ -69,8 +72,8 @@ MainWindow::MainWindow():
   Settings& settings = app->settings();
 
   ui.setupUi(this);
-  connect(ui.actionScreenshot, SIGNAL(triggered(bool)), app, SLOT(screenshot()));
-  connect(ui.actionPreferences, SIGNAL(triggered(bool)), app ,SLOT(editPreferences()));
+  connect(ui.actionScreenshot, &QAction::triggered, app, &Application::screenshot);
+  connect(ui.actionPreferences, &QAction::triggered, app , &Application::editPreferences);
 
   proxyModel_->addFilter(modelFilter_);
   proxyModel_->sort(Fm::FolderModel::ColumnFileName, Qt::AscendingOrder);
@@ -78,7 +81,7 @@ MainWindow::MainWindow():
 
   // build context menu
   ui.view->setContextMenuPolicy(Qt::CustomContextMenu);
-  connect(ui.view, SIGNAL(customContextMenuRequested(QPoint)), SLOT(onContextMenu(QPoint)));
+  connect(ui.view, &QWidget::customContextMenuRequested, this, &MainWindow::onContextMenu);
 
   // install an event filter on the image view
   ui.view->installEventFilter(this);
@@ -106,11 +109,11 @@ MainWindow::MainWindow():
 
   // create shortcuts
   QShortcut* shortcut = new QShortcut(Qt::Key_Left, this);
-  connect(shortcut, SIGNAL(activated()), SLOT(on_actionPrevious_triggered()));
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::on_actionPrevious_triggered);
   shortcut = new QShortcut(Qt::Key_Right, this);
-  connect(shortcut, SIGNAL(activated()), SLOT(on_actionNext_triggered()));
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::on_actionNext_triggered);
   shortcut = new QShortcut(Qt::Key_Escape, this);
-  connect(shortcut, SIGNAL(activated()), SLOT(onExitFullscreen()));
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::onExitFullscreen);
 }
 
 MainWindow::~MainWindow() {
@@ -173,7 +176,6 @@ void MainWindow::on_actionZoomOut_triggered() {
 }
 
 void MainWindow::onFolderLoaded(FmFolder* folder) {
-  qDebug("Finish loading: %d files", proxyModel_->rowCount());
   // if currently we're showing a file, get its index in the folder now
   // since the folder is fully loaded.
   if(currentFile_ && !currentIndex_.isValid()) {
@@ -184,6 +186,9 @@ void MainWindow::onFolderLoaded(FmFolder* folder) {
       thumbnailsView_->childView()->scrollTo(currentIndex_, QAbstractItemView::EnsureVisible);
     }
   }
+  // this is used to open the first image of a folder
+  else if (currentFile_ == nullptr)
+    on_actionFirst_triggered();
 }
 
 void MainWindow::openImageFile(QString fileName) {
@@ -193,9 +198,31 @@ void MainWindow::openImageFile(QString fileName) {
     fm_path_unref(path);
     return;
   }
-  // load the image file asynchronously
-  loadImage(path);
-  loadFolder(fm_path_get_parent(path));
+  if (QFileInfo(fileName).isDir()) {
+    if(fm_path_equal(path, folderPath_)) {
+      fm_path_unref(path);
+      return;
+    }
+    QList<QByteArray> formats = QImageReader::supportedImageFormats();
+    QStringList formatsFilters;
+    for (const QByteArray& format: formats)
+      formatsFilters << QString("*.") + format;
+    QDir dir(fileName);
+    dir.setNameFilters(formatsFilters);
+    dir.setFilter(QDir::Files | QDir::NoDotAndDotDot);
+    if(dir.entryList().isEmpty()) {
+      fm_path_unref(path);
+      return;
+    }
+    if(currentFile_)
+      fm_path_unref(currentFile_);
+    currentFile_ = nullptr;
+    loadFolder(path);
+  } else {
+    // load the image file asynchronously
+    loadImage(path);
+    loadFolder(fm_path_get_parent(path));
+  }
   fm_path_unref(path);
 }
 
@@ -206,14 +233,14 @@ void MainWindow::pasteImage(QImage newImage) {
   // cancel loading of current image
   if(loadJob_) {
     loadJob_->cancel(); // the job object will be freed automatically later
-    loadJob_ = NULL;
+    loadJob_ = nullptr;
   }
   setModified(true);
 
   currentIndex_ = QModelIndex(); // invaludate current index since we don't have a folder model now
   if(currentFile_)
     fm_path_unref(currentFile_);
-  currentFile_ = NULL;
+  currentFile_ = nullptr;
 
   image_ = newImage;
   ui.view->setImage(image_);
@@ -243,6 +270,12 @@ QString MainWindow::openFileName() {
   return fileName;
 }
 
+QString MainWindow::openDirectory() {
+  QString directory = QFileDialog::getExistingDirectory(this,
+          tr("Open directory"), QString());
+  return directory;
+}
+
 // popup a file dialog and retrieve the selected image file name
 QString MainWindow::saveFileName(QString defaultName) {
   QString filterStr;
@@ -275,6 +308,13 @@ void MainWindow::on_actionOpenFile_triggered() {
   }
 }
 
+void MainWindow::on_actionOpenDirectory_triggered() {
+  QString directory = openDirectory();
+  if(!directory.isEmpty()) {
+    openImageFile(directory);
+  }
+}
+
 void MainWindow::on_actionNewWindow_triggered() {
   Application* app = static_cast<Application*>(qApp);
   MainWindow* window = new MainWindow();
@@ -301,7 +341,13 @@ void MainWindow::on_actionSave_triggered() {
 void MainWindow::on_actionSaveAs_triggered() {
   if(saveJob_) // if we're currently saving another file
     return;
-  QString fileName = saveFileName(currentFile_ ? Fm::Path(currentFile_).displayName() : QString());
+  QString baseName;
+  if(currentFile_) {
+    char* dispName = fm_path_display_name(currentFile_, false);
+    baseName = QString::fromUtf8(dispName);
+    g_free(dispName);
+  }
+  QString fileName = saveFileName(baseName);
   if(!fileName.isEmpty()) {
     FmPath* path = fm_path_new_for_str(qPrintable(fileName));
     // save the image file asynchronously
@@ -353,7 +399,6 @@ void MainWindow::on_actionNext_triggered() {
     FmFileInfo* info = proxyModel_->fileInfoFromIndex(index);
     if(info) {
       FmPath* path = fm_file_info_get_path(info);
-      qDebug("try load: %s", fm_path_get_basename(path));
       loadImage(path, index);
     }
   }
@@ -364,7 +409,6 @@ void MainWindow::on_actionPrevious_triggered() {
     return;
   if(currentIndex_.isValid()) {
     QModelIndex index;
-    qDebug("current row: %d", currentIndex_.row());
     if(currentIndex_.row() > 0)
       index = proxyModel_->index(currentIndex_.row() - 1, 0);
     else
@@ -372,7 +416,6 @@ void MainWindow::on_actionPrevious_triggered() {
     FmFileInfo* info = proxyModel_->fileInfoFromIndex(index);
     if(info) {
       FmPath* path = fm_file_info_get_path(info);
-      qDebug("try load: %s", fm_path_get_basename(path));
       loadImage(path, index);
     }
   }
@@ -384,7 +427,6 @@ void MainWindow::on_actionFirst_triggered() {
     FmFileInfo* info = proxyModel_->fileInfoFromIndex(index);
     if(info) {
       FmPath* path = fm_file_info_get_path(info);
-      qDebug("try load: %s", fm_path_get_basename(path));
       loadImage(path, index);
     }
   }
@@ -395,8 +437,7 @@ void MainWindow::on_actionLast_triggered() {
   if(index.isValid()) {
     FmFileInfo* info = proxyModel_->fileInfoFromIndex(index);
     if(info) {
-      FmPath* path = fm_file_info_get_path(info);
-      qDebug("try load: %s", fm_path_get_basename(path));
+      FmPath* path = fm_file_info_get_path(info);;
       loadImage(path, index);
     }
   }
@@ -422,7 +463,7 @@ void MainWindow::loadFolder(FmPath* newFolderPath) {
 
 // the image is loaded (the method is only called if the loading is not cancelled)
 void MainWindow::onImageLoaded(LoadImageJob* job) {
-  loadJob_ = NULL; // the job object will be freed later automatically
+  loadJob_ = nullptr; // the job object will be freed later automatically
 
   image_ = job->image();
   ui.view->setAutoZoomFit(true);
@@ -437,18 +478,21 @@ void MainWindow::onImageLoaded(LoadImageJob* job) {
     // if there are errors
     // TODO: show a info bar?
   }
+
+  /* we resized and moved the window without showing
+     it in updateUI(), so we need to show it here */
+  show();
 }
 
 void MainWindow::onImageSaved(SaveImageJob* job) {
   if(!job->error()) {
     setModified(false);
   }
-  saveJob_ = NULL;
+  saveJob_ = nullptr;
 }
 
 // filter events of other objects, mainly the image view.
 bool MainWindow::eventFilter(QObject* watched, QEvent* event) {
-  // qDebug() << event;
   if(watched == ui.view) { // we got an event for the image view
     switch(event->type()) {
       case QEvent::Wheel: { // mouse wheel event
@@ -522,28 +566,56 @@ void MainWindow::updateUI() {
   if(currentFile_) {
     char* dispName = fm_path_display_basename(currentFile_);
     if(loadJob_) { // if loading is in progress
-      title = tr("%1 (Loading...) - Image Viewer")
+      title = tr("[*]%1 (Loading...) - Image Viewer")
                 .arg(QString::fromUtf8(dispName));
     }
     else {
       if(image_.isNull()) {
-        title = tr("%1 (Failed to Load) - Image Viewer")
+        title = tr("[*]%1 (Failed to Load) - Image Viewer")
                   .arg(QString::fromUtf8(dispName));
       }
       else {
-        title = tr("%1 (%2x%3) - Image Viewer")
+        title = tr("[*]%1 (%2x%3) - Image Viewer")
                   .arg(QString::fromUtf8(dispName))
                   .arg(image_.width())
                   .arg(image_.height());
+        /* Here we try to implement the following behavior as far as possible:
+             (1) A minimum size of 400x400 is assumed;
+             (2) The window is scaled to fit the image;
+             (3) But for too big images, the window is scaled down;
+             (4) The window is centered on the screen. */
+        if (!isVisible()) {
+          /* To have a correct position, we should move the window BEFORE
+             it's shown but we also need to know the dimensions of its view.
+             Therefore, we use show() without really showing the window. */
+          setAttribute(Qt::WA_DontShowOnScreen);
+          show();
+          int scrollThickness = style()->pixelMetric(QStyle::PM_ScrollBarExtent);
+          QSize newSize = size() + image_.size() - ui.view->size() + QSize(scrollThickness, scrollThickness);
+          QRect ag = QApplication::desktop()->availableGeometry();
+          // since the window isn't decorated yet, we have to assume a max thickness for its frame
+          QSize maxFrame = QSize(50, 100);
+          if (newSize.width() > ag.width() - maxFrame.width() || newSize.height() > ag.height() - maxFrame.height())
+            newSize.scale (ag.width() - maxFrame.width(), ag.height() - maxFrame.height(), Qt::KeepAspectRatio);
+          // a minimum size of 400x400 is good
+          if (newSize.width() < 400) newSize.rwidth() = 400;
+          if (newSize.height() < 400 ) newSize.rheight() = 400;
+          move (ag.x() + (ag.width() - newSize.width())/2,
+                ag.y() + (ag.height() - newSize.height())/2);
+          resize(newSize);
+          hide(); // hide it to show it again later, at onImageLoaded()
+          setAttribute(Qt::WA_DontShowOnScreen, false);
+        }
       }
     }
     g_free(dispName);
     // TODO: update status bar, show current index in the folder
   }
   else {
-    title = tr("Image Viewer");
+    title = tr("[*]Image Viewer");
   }
   setWindowTitle(title);
+  setWindowModified(imageModified_);
 }
 
 // Load the specified image file asynchronously in a worker thread.
@@ -552,7 +624,7 @@ void MainWindow::loadImage(FmPath* filePath, QModelIndex index) {
   // cancel loading of current image
   if(loadJob_) {
     loadJob_->cancel(); // the job object will be freed automatically later
-    loadJob_ = NULL;
+    loadJob_ = nullptr;
   }
   if(imageModified_) {
     // TODO: ask the user to save the modified image?
@@ -567,11 +639,34 @@ void MainWindow::loadImage(FmPath* filePath, QModelIndex index) {
   // clear current image, but do not update the view now to prevent flickers
   image_ = QImage();
 
-  // start a new gio job to load the specified image
-  loadJob_ = new LoadImageJob(this, filePath);
-  loadJob_->start();
+  const char* basename = fm_path_get_basename(currentFile_);
+  char* mime_type = g_content_type_guess(basename, NULL, 0, NULL);
+  QString mimeType;
+  if (mime_type) {
+    mimeType = QString(mime_type);
+    g_free(mime_type);
+  }
+  if(mimeType == "image/gif"
+     || mimeType == "image/svg+xml" || mimeType == "image/svg+xml-compressed") {
+    char *file_name = fm_path_to_str(currentFile_);
+    QString fileName(file_name);
+    g_free(file_name);
+    ui.view->setAutoZoomFit(true); // like in onImageLoaded()
+    if(mimeType == "image/gif")
+      ui.view->setGifAnimation(fileName);
+    else
+      ui.view->setSVG(fileName);
+    image_ = ui.view->image();
+    updateUI();
+    show();
+  }
+  else {
+    // start a new gio job to load the specified image
+    loadJob_ = new LoadImageJob(this, filePath);
+    loadJob_->start();
 
-  updateUI();
+    updateUI();
+  }
 }
 
 void MainWindow::saveImage(FmPath* filePath) {
@@ -583,24 +678,57 @@ void MainWindow::saveImage(FmPath* filePath) {
   // FIXME: add a cancel button to the UI? update status bar?
 }
 
+QGraphicsItem* MainWindow::getGraphicsItem() {
+  if(!ui.view->items().isEmpty())
+    return ui.view->items().at(0);
+  return nullptr;
+}
+
 void MainWindow::on_actionRotateClockwise_triggered() {
+  QGraphicsItem *graphItem = getGraphicsItem();
+  bool isGifOrSvg (graphItem->isWidget() // we have gif animation
+                   || dynamic_cast<QGraphicsSvgItem*>(graphItem)); // an SVG image;
   if(!image_.isNull()) {
     QTransform transform;
     transform.rotate(90.0);
     image_ = image_.transformed(transform, Qt::SmoothTransformation);
-    ui.view->setImage(image_);
+    /* when this is GIF or SVG, we need to rotate its corresponding QImage
+       without showing it to have the right measure for auto-zooming */
+    ui.view->setImage(image_, isGifOrSvg ? false : true);
     setModified(true);
   }
+
+  if(isGifOrSvg) {
+    QTransform transform;
+    transform.translate(graphItem->sceneBoundingRect().height(), 0);
+    transform.rotate(90);
+    // we need to apply transformations in the reverse order
+    QTransform prevTrans = graphItem->transform();
+    graphItem->setTransform(transform, false);
+    graphItem->setTransform(prevTrans, true);
+  }
 }
 
 void MainWindow::on_actionRotateCounterclockwise_triggered() {
+  QGraphicsItem *graphItem = getGraphicsItem();
+  bool isGifOrSvg (graphItem->isWidget()
+                   || dynamic_cast<QGraphicsSvgItem*>(graphItem));
   if(!image_.isNull()) {
     QTransform transform;
     transform.rotate(-90.0);
     image_ = image_.transformed(transform, Qt::SmoothTransformation);
-    ui.view->setImage(image_);
+    ui.view->setImage(image_, isGifOrSvg ? false : true);
     setModified(true);
   }
+
+  if(isGifOrSvg) {
+    QTransform transform;
+    transform.translate(0, graphItem->sceneBoundingRect().width());
+    transform.rotate(-90);
+    QTransform prevTrans = graphItem->transform();
+    graphItem->setTransform(transform, false);
+    graphItem->setTransform(prevTrans, true);
+  }
 }
 
 void MainWindow::on_actionCopy_triggered() {
@@ -626,26 +754,44 @@ void MainWindow::on_actionPaste_triggered() {
 }
 
 void MainWindow::on_actionFlipVertical_triggered() {
+  bool hasQGraphicsItem(false);
+  if(QGraphicsItem *graphItem = getGraphicsItem()) {
+    hasQGraphicsItem = true;
+    QTransform transform;
+    transform.scale(1, -1);
+    transform.translate(0, -graphItem->sceneBoundingRect().height());
+    QTransform prevTrans = graphItem->transform();
+    graphItem->setTransform(transform, false);
+    graphItem->setTransform(prevTrans, true);
+  }
   if(!image_.isNull()) {
     image_ = image_.mirrored(false, true);
-    ui.view->setImage(image_);
+    ui.view->setImage(image_, !hasQGraphicsItem);
     setModified(true);
   }
-  setModified(true);
 }
 
 void MainWindow::on_actionFlipHorizontal_triggered() {
+  bool hasQGraphicsItem(false);
+  if(QGraphicsItem *graphItem = getGraphicsItem()) {
+    hasQGraphicsItem = true;
+    QTransform transform;
+    transform.scale(-1, 1);
+    transform.translate(-graphItem->sceneBoundingRect().width(), 0);
+    QTransform prevTrans = graphItem->transform();
+    graphItem->setTransform(transform, false);
+    graphItem->setTransform(prevTrans, true);
+  }
   if(!image_.isNull()) {
-    image_ = image_.mirrored(true, false);
-    ui.view->setImage(image_);
+    image_ = image_.mirrored(true, true);
+    ui.view->setImage(image_, !hasQGraphicsItem);
     setModified(true);
   }
-  setModified(true);
 }
 
 void MainWindow::setModified(bool modified) {
   imageModified_ = modified;
-  updateUI(); // TODO: update title bar to reflect the state change
+  updateUI();
 }
 
 void MainWindow::applySettings() {
@@ -667,11 +813,9 @@ void MainWindow::on_actionPrint_triggered() {
     QRect pageRect = printer.pageRect();
     int cols = (image_.width() / pageRect.width()) + (image_.width() % pageRect.width() ? 1 : 0);
     int rows = (image_.height() / pageRect.height()) + (image_.height() % pageRect.height() ? 1 : 0);
-    // qDebug() << "page:" << printer.pageRect() << "image:" << image_.size() << "cols, rows:" << cols << rows;
     for(int row = 0; row < rows; ++row) {
       for(int col = 0; col < cols; ++col) {
         QRect srcRect(pageRect.width() * col, pageRect.height() * row, pageRect.width(), pageRect.height());
-        // qDebug() << "row:" << row << "col:" << col << "src:" << srcRect << "page:" << printer.pageRect();
         painter.drawImage(QPoint(0, 0), image_, srcRect);
         if(col + 1 == cols && row + 1 == rows) // this is the last page
           break;
@@ -695,7 +839,7 @@ void MainWindow::on_actionSlideShow_triggered(bool checked) {
     if(!slideShowTimer_) {
       slideShowTimer_ = new QTimer();
       // switch to the next image when timeout
-      connect(slideShowTimer_, SIGNAL(timeout()), SLOT(on_actionNext_triggered()));
+      connect(slideShowTimer_, &QTimer::timeout, this, &MainWindow::on_actionNext_triggered);
     }
     Application* app = static_cast<Application*>(qApp);
     slideShowTimer_->start(app->settings().slideShowInterval() * 1000);
@@ -705,7 +849,7 @@ void MainWindow::on_actionSlideShow_triggered(bool checked) {
   else {
     if(slideShowTimer_) {
       delete slideShowTimer_;
-      slideShowTimer_ = NULL;
+      slideShowTimer_ = nullptr;
       ui.actionSlideShow->setIcon(QIcon::fromTheme("media-playback-start"));
     }
   }
@@ -731,15 +875,23 @@ void MainWindow::setShowThumbnails(bool show) {
       thumbnailsView_->setFixedHeight(listView->gridSize().height() + scrollHeight);
       thumbnailsView_->setModel(proxyModel_);
       proxyModel_->setShowThumbnails(true);
-      connect(thumbnailsView_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(onThumbnailSelChanged(QItemSelection,QItemSelection)));
+      if (currentFile_) { // select the loaded image
+        currentIndex_ = indexFromPath(currentFile_);
+        listView->setCurrentIndex(currentIndex_);
+        // wait to center the selection
+        QCoreApplication::processEvents();
+        listView->scrollTo(currentIndex_, QAbstractItemView::PositionAtCenter);
+      }
+      connect(thumbnailsView_->selectionModel(), &QItemSelectionModel::selectionChanged,
+              this, &MainWindow::onThumbnailSelChanged);
     }
   }
   else {
     if(thumbnailsDock_) {
       delete thumbnailsView_;
-      thumbnailsView_ = NULL;
+      thumbnailsView_ = nullptr;
       delete thumbnailsDock_;
-      thumbnailsDock_ = NULL;
+      thumbnailsDock_ = nullptr;
     }
     proxyModel_->setShowThumbnails(false);
   }
@@ -769,6 +921,7 @@ void MainWindow::changeEvent(QEvent* event) {
           addAction(action);
       }
       addActions(ui.menubar->actions());
+      ui.view->hideCursor(true);
     }
     else { // restore to normal window mode
       ui.view->setFrameStyle(QFrame::StyledPanel|QFrame::Sunken);
@@ -783,6 +936,7 @@ void MainWindow::changeEvent(QEvent* event) {
       ui.statusBar->show();
       if(thumbnailsDock_)
         thumbnailsDock_->show();
+      ui.view->hideCursor(false);
     }
   }
   QWidget::changeEvent(event);
